// Copyright 2016 Dietrich Epp.
// This file is part of the Moria Gamedev Repository.  This file is licensed
// under the terms of the MIT license.  For more information, see LICENSE.txt.
'use strict';
var util = require('./util');
var Scale = util.Scale;

var Speed = 300;
var Jump = 240;

// We can leave the ground for this long and still jump.
var GroundTime = 100;
// Jump thrust lasts for this long.
var ThrustTime = 300;
// Can jump after this much time has passed.
var CooldownTime = 500;

var FrameTime = 50;
var StandFrames = ['p.stand'];
var JumpFrames = ['p.walk2'];
var WalkFrames = ['p.walk1', 'p.walk2', 'p.walk3', 'p.walk4'];
var StepTime = 300;

function Player(od, group, layers) {
	var player = Game.add.sprite(
		od.x * 32 + 16,
		od.y * 32 - 16,
		'tile',
		'blank',
		group);
	player.anchor.set(0.5, 0.5);
	this.sprite = player;
	Game.physics.p2.enable(player);
	player.body.setCircle(16);
	function addPart(frame) {
		var part = Game.make.sprite(0, 0, 'tile', frame);
		part.anchor.set(0.5, 0.5);
		player.addChild(part);
		return part;
	}
	this.head = addPart('p.head');
	this.legs = addPart('p.stand');
	this.eyes = addPart('eye1');
	this.hair = addPart('p.hair1');
	layers.player.push(this.sprite);
	var body = player.body;
	// player.body.collideWorldBounds = true;
	body.fixedRotation = true;
	this.input = Game.input.keyboard.createCursorKeys();
	this.jump = Game.input.keyboard.addKey(Phaser.Keyboard.SPACEBAR);
	this.groundTime = -1;
	this.thrustTime = -1;
	this.cooldownTime = -1;
	Game.camera.follow(this.sprite, Phaser.Camera.FOLLOW_PLATFORMER);
	this.freezeCount = 0;
	this.frameTime = -1;
	this.stepTime = -1;
	this.frame = 0;
	this.face = 1;
}

Player.prototype.create = function() {};

Player.prototype.update = function() {
	var now = Game.time.now;
	if (this.checkGroundCollision()) {
		this.groundTime = now + GroundTime;
	}
	var dx = 0, dy = 0;
	if (!this.freezeCount) {
		if (this.input.left.isDown) { dx--; }
		if (this.input.right.isDown) { dx++; }
		if (this.input.up.isDown || this.jump.isDown) { dy--; }
		if (this.input.down.isDown) { dy++; }
	}
	if (dx < 0) { this.sprite.body.moveLeft(Speed); }
	else if (dx > 0) { this.sprite.body.moveRight(Speed); }
	else { this.sprite.body.velocity.x = 0; }
	if (now <= this.thrustTime) {
		if (dy < 0) {
			this.sprite.body.moveUp(Jump);
			this.cooldownTime = now + CooldownTime;
		}
	} else if (now > this.cooldownTime) {
		if (dy < 0 && now <= this.groundTime) {
			this.sprite.body.moveUp(Jump);
			this.cooldownTime = now + CooldownTime;
			this.thrustTime = now + ThrustTime;
		}
	}
	var frame, frames, face, offset = 0;
	if (now > this.frameTime) {
		this.frame++;
		this.frameTime = now + FrameTime;
	}
	if (now < this.groundTime) {
		this.hair.frameName = 'p.hair1';
	} else {
		this.hair.frameName = 'p.hair2';
	}
	if (dx) {
		face = dx;
		if (now < this.groundTime) {
			frames = WalkFrames;
			if (this.frame === 1 || this.frame === 2) {
				offset = -4;
			}
			if (now >= this.stepTime) {
				Game.sound.play('sfx/step' + (1 + ((Math.random() * 3) | 0)), 0.3);
				this.stepTime = now + StepTime;
			}
		} else {
			frames = JumpFrames;
		}
	} else {
		face = this.face;
		frames = StandFrames;
	}
	if (face != this.face) {
		this.face = face;
		this.eyes.scale.set(face, +1);
		this.legs.scale.set(face, +1);
	}
	if (this.frame >= frames.length) {
		this.frame = 0;
	}
	this.legs.frameName = frames[this.frame];
	this.legs.y = offset;
	this.eyes.y = offset;
	this.head.y = offset;
	this.hair.y = offset;
};

Player.prototype.checkGroundCollision = function() {
	var isGrounded = false;
	var eqs = Game.physics.p2.world.narrowphase.contactEquations;
	var bdata = this.sprite.body.data;
	for (var i = 0; i < eqs.length; i++) {
		var eq = eqs[i];
		var a, other;
		if (eq.bodyA === bdata) {
			a = -eq.normalA[1];
			other = eq.bodyB;
		} else if (eq.bodyB === bdata) {
			a = eq.normalA[1];
			other = eq.bodyA;
		} else {
			continue;
		}
		if (a > 0.8) {
			isGrounded = true;
			var parent = other.parent;
			if (parent) {
				var sprite = parent.sprite;
				if (sprite) {
					var onPress = sprite.data.onPress;
					if (onPress) {
						onPress();
					}
				}
			}
		}
	}
	return isGrounded;
};

Player.prototype.freeze = function(freeze) {
	if (freeze) {
		this.freezeCount++;
	} else {
		this.freezeCount--;
		if (this.freezeCount < 0) {
			this.freezeCount = 0;
		}
	}
};

module.exports = {
	Player: Player,
};
